//****************************************************************************
//               Fourth-order Runge-Kutta for the Rössler system
//
//                                      Les Monts du Roumois, October 31, 2019
//****************************************************************************
# include <cstdlib>
# include <iostream>
# include <fstream>
# include <iomanip>
# include <cmath>
# include <ctime>

using namespace std;

//const double a = 1.0;
//const double b = 15.00;
//const double b = 12.1574;                     
// ---------------------------------------------------------------------------
//const double a = 1.0;                                   // Burke & Shaw type
//const double b = 8.261;
//const double c = 0.05;
// ---------------------------- Burke & Shaw attractor -----------------------
//const double a = 2.0;
//const double b = 25.18;
//const double c = 0.05;
// -----------------------------Lorenz attractor -----------------------------
//const double a = 11.0;
//const double b = 270.0/11.0;
//const double c = 8.0/3.0;  
//const double dt = 0.01; 
// -----------------------------Burke & Shaw ---------------------------------
  const double a = 15.7975;
  const double b = 40.0;
  const double c = 1.0;
  const double dt = 0.01;        

  const int m = 3;                                    // state space dimension 
// **************** Liu & Yang system ****************************************
#define fx(x,y,z) - a*x + a*y
#define fy(x,y,z) b*x - x*z
#define fz(x,y,z) - c*z + x*y

void Runge(double *v);                             // Fourth-order Runge-Kutta

int main ( )
{
  int j;
  const int Ntrans = 90000;
  const int Npt = Ntrans + 7000;                      // number of data points 
  double *v, Y, Z; 

  v = new double[m];
//****************************************************************************
  ofstream data_12 ("lorenz.dat");
  cout << " a = " << a << " b = " << b << " c = " << c << "\n";

  v[0] = 0.1;
  v[1] =-0.1;
  v[2] = 0.1;
  for (j = 0; j < Npt; j++)
  {
    Runge(v);
    if ((j > Ntrans) && (j % 1 == 0))
    {
//    data_12 << v[0] << " " << v[1] << " " << v[2] << "\n";
      data_12 << v[0] << " " << v[2] << " " << v[2] << "\n";
    }
  }
  data_12.close ( );
  system("xmgrace lorenz.dat -param loratt.par"); 
  return 0;
}
// ***************************************************************************
   void Runge(double *v)                           // Fourth-order Runge-Kutta
// ***************************************************************************
   {
      int j;
      double phi[m], savev[m], FF[m];

      FF[0] = fx(v[0],v[1],v[2]);
      FF[1] = fy(v[0],v[1],v[2]);
      FF[2] = fz(v[0],v[1],v[2]);
      for (j=0; j <m; j++)
      {
        savev[j]=v[j];
        phi[j]=FF[j];
        v[j]=savev[j]+0.5*dt*FF[j];
      }
      FF[0] = fx(v[0],v[1],v[2]);
      FF[1] = fy(v[0],v[1],v[2]);
      FF[2] = fz(v[0],v[1],v[2]);
      for (j=0; j <m; j++)
      {
        phi[j] = phi[j] + 2.0*FF[j];
        v[j]=savev[j] + 0.5*dt*FF[j];
      }
      FF[0] = fx(v[0],v[1],v[2]);
      FF[1] = fy(v[0],v[1],v[2]);
      FF[2] = fz(v[0],v[1],v[2]);
      for (j=0; j <m; j++)
      {
        phi[j] = phi[j] + 2.0*FF[j];
        v[j] = savev[j] + dt*FF[j];
      }
      FF[0] = fx(v[0],v[1],v[2]);
      FF[1] = fy(v[0],v[1],v[2]);
      FF[2] = fz(v[0],v[1],v[2]);
      for (j=0; j <m; j++)
      {
        v[j] = savev[j] + (phi[j] + FF[j])*dt/6.0;
      }
   }
// ***************************************************************************
